From: Greg KH <gregkh@suse.de>
To: linux-kernel@vger.kernel.org, stable@kernel.org
Cc: stable-review@kernel.org, torvalds@linux-foundation.org,
akpm@linux-foundation.org, alan@lxorguk.ukuu.org.uk,
Alan Stern <stern@rowland.harvard.edu>
Subject: [20/26] USB: EHCI: unlink unused QHs when the controller is stopped
Date: Tue, 19 Apr 2011 14:02:36 -0700 [thread overview]
Message-ID: <20110419210324.385138488@clark.kroah.org> (raw)
In-Reply-To: <20110419210333.GA17417@kroah.com>
2.6.33-longterm review patch. If anyone has any objections, please let us know.
------------------
From: Alan Stern <stern@rowland.harvard.edu>
commit 94ae4976e253757e9b03a44d27d41b20f1829d80 upstream.
This patch (as1458) fixes a problem affecting ultra-reliable systems:
When hardware failover of an EHCI controller occurs, the data
structures do not get released correctly. This is because the routine
responsible for removing unused QHs from the async schedule assumes
the controller is running properly (the frame counter is used in
determining how long the QH has been idle) -- but when a failover
causes the controller to be electronically disconnected from the PCI
bus, obviously it stops running.
The solution is simple: Allow scan_async() to remove a QH from the
async schedule if it has been idle for long enough _or_ if the
controller is stopped.
Signed-off-by: Alan Stern <stern@rowland.harvard.edu>
Reported-and-Tested-by: Dan Duval <dan.duval@stratus.com>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>
---
drivers/usb/host/ehci-q.c | 15 +++++++++------
1 file changed, 9 insertions(+), 6 deletions(-)
--- a/drivers/usb/host/ehci-q.c
+++ b/drivers/usb/host/ehci-q.c
@@ -1246,24 +1246,27 @@ static void start_unlink_async (struct e
static void scan_async (struct ehci_hcd *ehci)
{
+ bool stopped;
struct ehci_qh *qh;
enum ehci_timer_action action = TIMER_IO_WATCHDOG;
ehci->stamp = ehci_readl(ehci, &ehci->regs->frame_index);
timer_action_done (ehci, TIMER_ASYNC_SHRINK);
rescan:
+ stopped = !HC_IS_RUNNING(ehci_to_hcd(ehci)->state);
qh = ehci->async->qh_next.qh;
if (likely (qh != NULL)) {
do {
/* clean any finished work for this qh */
- if (!list_empty (&qh->qtd_list)
- && qh->stamp != ehci->stamp) {
+ if (!list_empty(&qh->qtd_list) && (stopped ||
+ qh->stamp != ehci->stamp)) {
int temp;
/* unlinks could happen here; completion
* reporting drops the lock. rescan using
* the latest schedule, but don't rescan
* qhs we already finished (no looping).
*/
qh = qh_get (qh);
qh->stamp = ehci->stamp;
@@ -1284,9 +1287,9 @@ rescan:
*/
if (list_empty(&qh->qtd_list)
&& qh->qh_state == QH_STATE_LINKED) {
- if (!ehci->reclaim
- && ((ehci->stamp - qh->stamp) & 0x1fff)
- >= (EHCI_SHRINK_FRAMES * 8))
+ if (!ehci->reclaim && (stopped ||
+ ((ehci->stamp - qh->stamp) & 0x1fff)
+ >= EHCI_SHRINK_FRAMES * 8))
start_unlink_async(ehci, qh);
else
action = TIMER_ASYNC_SHRINK;
next prev parent reply other threads:[~2011-04-19 21:04 UTC|newest]
Thread overview: 27+ messages / expand[flat|nested] mbox.gz Atom feed top
2011-04-19 21:03 [00/26] 2.6.33.12-longterm review Greg KH
2011-04-19 21:02 ` [01/26] cifs: always do is_path_accessible check in cifs_mount Greg KH
2011-04-19 21:02 ` [02/26] [media] video: sn9c102: world-wirtable sysfs files Greg KH
2011-04-19 21:02 ` [03/26] UBIFS: restrict world-writable debugfs files Greg KH
2011-04-19 21:02 ` [04/26] NET: cdc-phonet, handle empty phonet header Greg KH
2011-04-19 21:02 ` [05/26] x86: Fix a bogus unwind annotation in lib/semaphore_32.S Greg KH
2011-04-19 21:02 ` [06/26] [IA64] tioca: Fix assignment from incompatible pointer warnings Greg KH
2011-04-19 21:02 ` [07/26] [IA64] mca.c: Fix cast from integer to pointer warning Greg KH
2011-04-19 21:02 ` [08/26] ramfs: fix memleak on no-mmu arch Greg KH
2011-04-19 21:02 ` [09/26] MAINTAINERS: update STABLE BRANCH info Greg KH
2011-04-19 21:02 ` [10/26] UBIFS: fix oops when R/O file-system is fsynced Greg KH
2011-04-19 21:02 ` [11/26] x86, AMD: Set ARAT feature on AMD processors Greg KH
2011-04-19 21:02 ` [12/26] x86, cpu: AMD errata checking framework Greg KH
2011-04-19 21:02 ` [13/26] x86, cpu: Clean up AMD erratum 400 workaround Greg KH
2011-04-19 21:02 ` [14/26] x86, amd: Disable GartTlbWlkErr when BIOS forgets it Greg KH
2011-04-19 21:02 ` [15/26] USB: ftdi_sio: Added IDs for CTI USB Serial Devices Greg KH
2011-04-19 21:02 ` [16/26] USB: ftdi_sio: add PID for OCT DK201 docking station Greg KH
2011-04-19 21:02 ` [17/26] USB: ftdi_sio: add ids for Hameg HO720 and HO730 Greg KH
2011-04-19 21:02 ` [18/26] next_pidmap: fix overflow condition Greg KH
2011-04-19 21:02 ` [19/26] proc: do proper range check on readdir offset Greg KH
2011-04-19 21:02 ` Greg KH [this message]
2011-04-19 21:02 ` [21/26] USB: fix formatting of SuperSpeed endpoints in /proc/bus/usb/devices Greg KH
2011-04-19 21:02 ` [22/26] USB: xhci - fix unsafe macro definitions Greg KH
2011-04-19 21:02 ` [23/26] USB: xhci - fix math in xhci_get_endpoint_interval() Greg KH
2011-04-19 21:02 ` [24/26] x86, cpu: Fix regression in AMD errata checking code Greg KH
2011-04-19 21:02 ` [25/26] net: ax25: fix information leak to userland harder Greg KH
2011-04-19 21:02 ` [26/26] net: fix rds_iovec page count overflow Greg KH
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20110419210324.385138488@clark.kroah.org \
--to=gregkh@suse.de \
--cc=akpm@linux-foundation.org \
--cc=alan@lxorguk.ukuu.org.uk \
--cc=linux-kernel@vger.kernel.org \
--cc=stable-review@kernel.org \
--cc=stable@kernel.org \
--cc=stern@rowland.harvard.edu \
--cc=torvalds@linux-foundation.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -1246,24 +1246,27 @@ static void start_unlink_async (struct e
* reporting drops the lock. rescan using
* the latest schedule, but don't rescan
- * qhs we already finished (no looping).
+ * qhs we already finished (no looping)
+ * unless the controller is stopped.
*/
qh = qh_get (qh);
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
all inboxes | Powered by JetHome®